import matplotlib.pyplot as plt
from matplotlib.patches import RegularPolygon
import numpy as np
from .distance2detelements import distance2detelements
from .detectors import detector_element_coordinates
from ..tools.cast_data import cast_data
from ..tools.color_from_map import color_from_map
      

def plot_fingerprint_airyscan(counts, cmap='inferno', plot=False, figsize=(5,5)):
    """
    Plot the airyscan fingerprint from a data set with the photon counts for
    each of the 32 detector elements.
    The elements in counts are plotted in the following way:
    
            22  21
      23  10  9   8  20
    24  11  2   1   7  19
      12  3   0   6  18  31
    25  13  4   5  17  30
      26 14  15  16  29
           27  28

    Parameters
    ----------
    counts : np.array
        Array with the 32 photon counts of the airyscan.
        Channel numbers according to Zeiss (ch 0 = center)
    plot : boolean, optional
        Plot the finger print

    Returns
    -------
    sx, sy : list
        x and y coordinates of the hexagons
    c : np.array()
        2D array with RGBA values for each hexagon

    """
    
    sx = detector_element_coordinates('airyscan')
    n_elements= len(sx)
    
    cmin = np.min(counts)
    cmax = np.max(counts)
    if cmax == cmin:
        cmax = cmin + 1
    color_code = np.zeros((n_elements, 3))
    
    for i in range(n_elements):
        color_code[i,:] = color_from_map(counts[i], startv=cmin, stopv=cmax, cmap=cmap)
    
    if plot:
        plt.figure(figsize=figsize)
        ax = plt.subplot(1,1,1)
        for x, y, c in zip(sx[:,0], sx[:,1], color_code):
            color = c
            ax.add_patch(RegularPolygon((x, y),
                                        numVertices=6,
                                        radius=0.55, 
                                        orientation=np.radians(120),
                                        facecolor = color,
                                        alpha=1))
        
        plt.xlim([-3,3.5])
        plt.ylim([-3.5,3.5])
        plt.xticks([])
        plt.yticks([])
        ax.set_axis_off()
        ax.set_box_aspect(1)
    
    return list(sx[:,0]), list(sx[:,1]), color_code


def plot_fingerprint_luminosa(counts, cmap='inferno', plot=False, figsize=(5,5)):
    """
    Plot the airyscan fingerprint from a data set with the photon counts for
    each of the 23 (or 32) detector elements.
    
    List order either

    31  30  29  28  27
      26  25  24  23
    22  21  20  19  18
      17  16  15  14
    13  12  11  10  09


    Or
    
    22  21  20  19  18
      17  16  15  14
    13  12  11  10  09
      08  07  06  05
    04  03  02  01  00

    Parameters
    ----------
    counts : np.array
        1D array with the 23 photon counts of the luminosa finger print.
        Either 23 elements or 32 elements with first 9 elements 0
    plot : boolean, optional
        Plot the finger print

    Returns
    -------
    hexb : np.array()
        Array for hexbin plotting

    """
    
    sx = detector_element_coordinates('luminosa')
    sx = sx[9:,:]
    n_elements= len(sx)
    
    if len(counts) == 32:
        counts = counts[9:]
    
    cmin = np.min(counts)
    cmax = np.max(counts)
    if cmax == cmin:
        cmax = cmin + 1
    color_code = np.zeros((n_elements, 3))
    
    for i in range(n_elements):
        color_code[i,:] = color_from_map(counts[i], startv=cmin, stopv=cmax, cmap=cmap)
    
    # make a plot of the detector array
    if plot:
        plt.figure(figsize=figsize)
        ax = plt.subplot(1,1,1)
        for x, y, c in zip(sx[:,0], sx[:,1], color_code):
            color = c
            ax.add_patch(RegularPolygon((x, y),
                                        numVertices=6,
                                        radius=0.55, 
                                        orientation=np.radians(120),
                                        facecolor = color,
                                        alpha=1))
        
        plt.xlim([-3,3])
        plt.ylim([-3,3])
        plt.xticks([])
        plt.yticks([])
        ax.set_axis_off()
        ax.set_box_aspect(1)
    
    return list(sx[:,0]), list(sx[:,1]), color_code


def plot_fingerprint5x5(data, show_perc=True, dtype='int64', normalize=False, savefig=0, vminmax = 'auto'):
    """
    Make finger print plot of SPAD-fcs data with 25 channels.
    
     0  1  2  3  4
     5  6  7  8  9
    10 11 12 13 14
    15 16 17 18 19
    20 21 22 23 24

    Parameters
    ----------
    data : np.array()
        Nx26 or Nx25 array with the fcs data
        or data object with data.det0 etc. arrival times.
    show_perc : boolean, optional
        Show percentages. The default is True.
    dtype : string, optional
        Data type. The default is 'int64'.
    normalize : boolean, optional
        Convert total counts to average counts per bin if True. The default is False.
    savefig : int, optional
        Path to store figure. The default is 0.
    vminmax : vector or string, optional
        Vector with minimum and maximum color bar value. The default is 'auto'.

    Returns
    -------
    airy : np.array()
        26 element vector with the sum of the rows and plot.

    """
    
    if type(data) == np.ndarray:
        # data is numpy array with intensity traces
        if len(np.shape(data)) > 1:
            # if 2D array, convert to dtype and sum over all rows
            data = cast_data(data, dtype)
            airy = np.sum(data, axis=0)
        else:
            airy = data
        airy2 = airy[0:25]
    else:
        if hasattr(data, 'det24'):
            # data is fcs2arrivaltimes.ATimesData object with 25 elements
            airy2 = np.zeros(25)
            for det in range(25):
                airy2[det] = len(getattr(data, 'det' + str(det)))
        else:
            # data is fcs2arrivaltimes.ATimesData object with 21 elements
            airy2 = np.zeros(25)
            # *  0  1  2  *
            # 3  4  5  6  7
            # 8  9  10 11 12
            # 13 14 15 16 17
            # *  18 19 20 *
            dets = [1, 2, 3, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 23]
            for det in range(21):
                if hasattr(data, 'det' + str(det)):
                    airy2[dets[det]] = len(getattr(data, 'det' + str(det)))
                else:
                    airy2[dets[det]] = 0
        airy = airy2
    
    if normalize:
        airy2 = airy2 / np.size(data, 0)
    
    airyMax = np.max(airy2)
    airyMin = np.min(airy2)
    airyCentPerc = (0.2 * (airyMax - airyMin) + airyMin) / airyMax * 100
        
    airy2 = airy2.reshape(5, 5)
    
    
    plt.figure()
    fontSize = 20
    plt.rcParams.update({'font.size': fontSize})
    plt.rcParams['mathtext.rm'] = 'Arial'
    
    if vminmax == 'auto':
        plt.imshow(airy2, cmap='hot', interpolation='nearest')
    else:
        plt.imshow(airy2, cmap='hot', interpolation='nearest', vmin=vminmax[0], vmax=vminmax[1])
    ax = plt.gca()
    
    # Major ticks
    ax.set_xticks([])
    ax.set_yticks([])
    # Labels for major ticks
    ax.set_xticklabels([])
    ax.set_yticklabels([])
    # Minor ticks
    ax.set_xticks(np.arange(-0.5, 4.5, 1), minor=True)
    ax.set_yticks(np.arange(-0.5, 5.5, 1), minor=True)
    # Gridlines based on minor ticks
    #ax.grid(which='minor', color='w', linestyle='-', linewidth=1)
    ax.tick_params(axis=u'both', which=u'both',length=0)
    
    cbar = plt.colorbar()
    cbar.ax.tick_params(labelsize=fontSize)

    if type(show_perc) is str and show_perc=="numbers":
        for i in range(5):
            for j in range(5):
                if vminmax == 'auto':
                    perc = round(airy2[i, j] / airyMax * 100)
                else:
                    perc = round(airy2[i, j] / vminmax[1] * 100)
                c="k"
                if perc < airyCentPerc:
                    c="w"
                plt.text(j, i, '{:.1f}'.format(airy2[i, j]), ha="center", va="center", color=c, fontsize=18)    
    elif show_perc:
        for i in range(5):
            for j in range(5):
                if vminmax == 'auto':
                    perc = round(airy2[i, j] / airyMax * 100)
                else:
                    perc = round(airy2[i, j] / vminmax[1] * 100)
                c="k"
                if perc < airyCentPerc:
                    c="w"
                plt.text(j, i, '{:.0f}%'.format(perc), ha="center", va="center", color=c, fontsize=18)

    if savefig != 0:
        plt.tight_layout()
        if savefig[-3:] == 'svg':
            plt.rcParams['svg.fonttype'] = 'none'
        plt.savefig(savefig, format=savefig[-3:])

    return airy

def plot_det_dist():
    det = []
    for i in range(25):
        det.append(distance2detelements(i, 12))
    det = np.resize(det, (5, 5))
    plt.figure()
    plt.imshow(det, cmap='viridis')


import numpy as np
import re
from .extract_spad_data_kw import keyword_2_ch


def extract_spad_photon_streams(data, mode):
    """
    Extract data from specific detector elements from SPAD-fcs time-tagging data

    Parameters
    ----------
    data : object
        Object with for each detector element a 2D array [t, m, F1, F2]
        data.det0, data.det1, etc. with
        - t: vector with macrotimes photon arrivals (ps)
        - m: microtimes (ps)
        - F1, F2, ... filter weights (based on microtimes), if any.
    mode : string
        Either
        - "C3+4+7+12" to custom sum over channels 3, 4, 7 and 12
        - int: to extract photons from a single channel
        - string with a predefined keyword, such as "sum3"

    Returns
    -------
    datasum : np.array()
        2D array with concatenated and sorted detector elements data.

    """
    
    if isinstance(mode, str):
        if mode[0] == 'C':
            # list of channels to be summed, e.g. "C1+3+12+42"
            listOfFields = ['det' + str(i) for i in re.findall(r'\d+', mode)]
        else:
            # get list from dictionary
            listOfFields = keyword_2_ch[mode]
            listOfFields  = ['det' + str(i) for i in listOfFields]
    else:
        # number given
        listOfFields = ['det' + str(mode)]
   
    # concatenate all photon streams
    for j in range(len(listOfFields)):
        if j == 0:
            datasum = getattr(data, listOfFields[j])
        else:
            datasum = np.concatenate((datasum, getattr(data, listOfFields[j])))
    
    # sort all photon streams
    datasum = datasum[datasum[:,0].argsort()]
    
    # remove photon pairs with the same macrotime
    [dummy, ind] = np.unique(datasum[:,0], return_index=True)
    datasum = datasum[ind,:]
    
    return datasum



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 21 09:18:22 2025

@author: eslenders
"""

def print_table(table, width=8):
    """
    Print 2D numeric table in a nice tabular way

    Parameters
    ----------
    table : np.array()
        2D array.
    width : int, optional
        Width of each cell. The default is 8.

    Returns
    -------
    None.

    """
    for row in table:
        print("".join(f"{v:{width}.2f}" for v in row))



from .detectors import detector_element_coordinates
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.patches import CirclePolygon
from ..tools.color_from_map import color_from_map
from .extract_spad_data_kw import keyword_2_ch
import re


def plot_correlations_microimage(detector, list_of_g, list_of_g_out=None, averaging=None, figsize=(5,5), savefig=None):
    """
    Plot scheme of the detector with correlations indicated as colored pixels
    or arrows.
    Uses output from get_corr_input, e.g.
    list_of_g, list_of_g_out, averaging = get_corr_input('pair-correlation fcs', 'airyscan')
    plot_correlations_microimage('airyscan', list_of_g, list_of_g_out, averaging)

    Parameters
    ----------
    detector : str
        DESCRIPTION.
    list_of_g : list
        List of the correlations that are calculated.
    list_of_g_out : list of str, optional
        Names of the correlations. The default is None.
    averaging : list, optional
        List of which correlations are averaged. The default is None.
    figsize : tuple, optional
        Figure size. The default is (5,5).

    Returns
    -------
    Plot.

    """
    
    xx = detector_element_coordinates(detector, element=None)
    
    if averaging is None and (type(list_of_g[0]) is not str or str.lower(list_of_g[0]) != 'crossall'):
        
        if len(list_of_g) > 6:
            n_rows = int(np.ceil(len(list_of_g)/6))
            n_columns = 6
        else:
            n_rows = 1
            n_columns = len(list_of_g)
        
        fig, axs = plt.subplots(n_rows, n_columns, figsize=figsize, squeeze=False, constrained_layout=True)
        axs = np.atleast_1d(axs)
        used_ch = []
        
        if list_of_g_out is None:
            list_of_g_out = list_of_g
        
        for idx_scalar, corr in enumerate(list_of_g):
    
            row = idx_scalar // 6
            col = idx_scalar % 6
            ax = axs[row, col]
            clr = color_from_map(32, startv=0, stopv=41, cmap='inferno')
    
            # draw detector elements
            for c in range(len(xx)):
                ax.add_patch(CirclePolygon((xx[c, 0], xx[c, 1]), radius=0.45, resolution=51, alpha=1, color='grey'))
            ax.set_xlim([np.min(xx)-0.5, np.max(xx)+0.5])
            ax.set_ylim([np.min(xx)-0.5, np.max(xx)+0.5])        
    
            if type(corr) == int:
                # autocorrelation single channel
                ax.add_patch(CirclePolygon((xx[corr, 0], xx[corr, 1]), radius=0.45, resolution=51, alpha=1, color=clr))
                used_ch.append(xx[corr, :])
                
            elif corr[0] == 'x':
                # cross-correlation two channels, e.g., x0412 between 4 and 12
                c0 = int(corr[1:3]) # first channel
                c1 = int(corr[3:5]) # second channel
                used_ch.append(xx[c0, :])
                used_ch.append(xx[c1, :])
                if c0 == c1:
                    ax.add_patch(CirclePolygon((xx[c0, 0], xx[c0, 1]), radius=0.45, resolution=51, alpha=1, color=clr))
                else:
                    ax.add_patch(CirclePolygon((xx[c0, 0], xx[c0, 1]), radius=0.45, resolution=51, alpha=1, color=clr))
                    ax.add_patch(CirclePolygon((xx[c1, 0], xx[c1, 1]), radius=0.45, resolution=51, alpha=1, color=clr))
                    ax.arrow(xx[c0, 0], xx[c0, 1], xx[c1, 0]-xx[c0, 0], xx[c1, 1]-xx[c0, 1], width=0.1, length_includes_head=True, alpha=0.5, color='k', edgecolor=None)
                
            elif corr[0] == 'C':
                # crosscorrelation custom sum of channels
                xpos = np.max([corr.find('X'), corr.find('x')])
                if xpos > -1:
                    sum_ch = [int(i) for i in re.findall(r'\d+', corr[0:xpos])]
                    ch0_mean = np.mean(xx[sum_ch,:], 0)
                    for k in sum_ch:
                        used_ch.append(xx[k, :])
                        ax.add_patch(CirclePolygon((xx[k, 0], xx[k, 1]), radius=0.45, resolution=51, alpha=0.5, color=clr))
                    sum_ch = [int(i) for i in re.findall(r'\d+', corr[xpos:])]
                    ch1_mean = np.mean(xx[sum_ch,:], 0)
                    for k in sum_ch:
                        clr2 = color_from_map(32, startv=0, stopv=41, cmap='inferno')
                        ax.add_patch(CirclePolygon((xx[k, 0], xx[k, 1]), radius=0.45, resolution=51, alpha=1, color=clr2))
                        used_ch.append(xx[k, :])
                    ax.arrow(ch0_mean[0], ch0_mean[1], ch1_mean[0]-ch0_mean[0], ch1_mean[1]-ch0_mean[1], width=0.1, length_includes_head=True, alpha=0.5, color='k', edgecolor=None)
                    
                else:
                    sum_ch = [int(i) for i in re.findall(r'\d+', corr)]
                    for k in sum_ch:
                        ax.add_patch(CirclePolygon((xx[k, 0], xx[k, 1]), radius=0.45, resolution=51, alpha=1, color=clr))
                        used_ch.append(xx[k, :])
            
            else:
                # keyword
                sum_ch = keyword_2_ch[corr]
                for k in sum_ch:
                    ax.add_patch(CirclePolygon((xx[k, 0], xx[k, 1]), radius=0.45, resolution=51, alpha=1, color=clr))
                    used_ch.append(xx[k, :])
            
            plt.xticks([])
            plt.yticks([])
            ax.set_axis_off()
            ax.set_title(str(list_of_g_out[idx_scalar]), fontsize=9)
            ax.set_box_aspect(1)
        
        for idx_scalar, corr in enumerate(list_of_g):
            row = idx_scalar // 6
            col = idx_scalar % 6
            ax = axs[row, col]
            for i in xx:
                if not any(np.array_equal(i, pt) for pt in used_ch):
                    ax.add_patch(CirclePolygon((i[0], i[1]), radius=0.45, resolution=51, alpha=0.7, color='white'))
                
    else:
        #cross all
        if len(averaging) > 6:
            n_rows = int(np.ceil(len(averaging)/6))
            n_columns = 6
        else:
            n_rows = 1
            n_columns = len(averaging)
        
        fig, axs = plt.subplots(n_rows, n_columns, figsize=figsize, squeeze=False, constrained_layout=True)
        axs = np.atleast_1d(axs)
        used_ch = []
        
        if list_of_g_out is None:
            list_of_g_out = averaging
        
        for idx_scalar, av in enumerate(averaging):
            
            row = idx_scalar // 6
            col = idx_scalar % 6
            ax = axs[row, col]
            
            # draw detector elements
            for c in range(len(xx)):
                ax.add_patch(CirclePolygon((xx[c, 0], xx[c, 1]), radius=0.45, resolution=51, alpha=1, color='grey'))
            ax.set_xlim([np.min(xx)-0.5, np.max(xx)+0.5])
            ax.set_ylim([np.min(xx)-0.5, np.max(xx)+0.5])       
        
            all_ch = [int(ch_nr) for ch_nr in re.findall(r'\d+', av)]
            for j in range(int(len(all_ch)/2)):
                c0 = all_ch[2*j]
                c1 = all_ch[2*j+1]
                used_ch.append(xx[c0, :])
                used_ch.append(xx[c1, :])
                clr0 = color_from_map(32, startv=0, stopv=41, cmap='inferno')
                ax.add_patch(CirclePolygon((xx[c0, 0], xx[c0, 1]), radius=0.45, resolution=51, alpha=1, color=clr0))
                ax.add_patch(CirclePolygon((xx[c1, 0], xx[c1, 1]), radius=0.45, resolution=51, alpha=1, color=clr0))
            for j in range(int(len(all_ch)/2)):
                c0 = all_ch[2*j]
                c1 = all_ch[2*j+1]
                ax.arrow(xx[c0, 0], xx[c0, 1], xx[c1, 0]-xx[c0, 0], xx[c1, 1]-xx[c0, 1], width=0.1, length_includes_head=True, alpha=0.5, color='k', edgecolor=None)
        
            plt.xticks([])
            plt.yticks([])
            ax.set_axis_off()
            ax.set_title(str(list_of_g_out[idx_scalar]), fontsize=9)
            ax.set_box_aspect(1)
        
        for idx_scalar, corr in enumerate(averaging):
            row = idx_scalar // 6
            col = idx_scalar % 6
            ax = axs[row, col]
            for i in xx:
                if not any(np.array_equal(i, pt) for pt in used_ch):
                    ax.add_patch(CirclePolygon((i[0], i[1]), radius=0.45, resolution=51, alpha=0.7, color='white'))
    
    while idx_scalar < n_columns * n_rows:
        row = idx_scalar // 6
        col = idx_scalar % 6
        ax = axs[row, col]
        plt.xticks([])
        plt.yticks([])
        ax.set_axis_off()
        idx_scalar += 1
    
    if savefig is not None:
        plt.savefig(savefig)


def plot_det_element_map(detector, figsize=(5,5), savefig=None):
    xx = detector_element_coordinates(detector, element=None)
    
    fig, axs = plt.subplots(figsize=figsize, constrained_layout=True)

    # Identify which rows are (0,0)
    is_zero = np.all(xx == 0, axis=1)
    # Find the last zero-row index
    zero_indices = np.where(is_zero)[0]
    if len(zero_indices) > 0:
        last_zero = zero_indices[-1]
        is_zero[last_zero] = False
        mask = ~is_zero
    else:
        mask = np.ones(len(xx), dtype=bool)   # no zero rows
    
    # draw detector elements
    for c in range(len(xx)):
        x = xx[c, 0]
        y = xx[c, 1]
        clr = color_from_map(np.sqrt(x**2+y**2), startv=-1, stopv=4, cmap='inferno_r')
        if mask[c]:
            axs.add_patch(CirclePolygon((x, y), radius=0.45, resolution=51, alpha=0.7, color=clr))
            axs.text(xx[c, 0], xx[c, 1], str(c), fontsize=18, verticalalignment='center', horizontalalignment='center')
    axs.set_xlim([np.min(xx)-0.5, np.max(xx)+0.5])
    axs.set_ylim([np.min(xx)-0.5, np.max(xx)+0.5])        
            
    plt.xticks([])
    plt.yticks([])
    axs.set_axis_off()
    axs.set_title(detector, fontsize=18)
    axs.set_box_aspect(1)
        
    
    if savefig is not None:
        plt.savefig(savefig)


import numpy as np


def distance_between_elements(detector, elem1, elem2):
    """
    Calculate the distance between two SPAD detector elements in a.u.
    (nearest neighbours 1 unit)

    Parameters
    ----------
    detector : str
        Detector type: either 'airyscan', 'luminosa', or 'nsparc'
    elem1 : int
        Number of the first detector element.
    elem1 : int
        Number of the second detector element.

    Returns
    -------
    distance : float
        Distance between the two elements.

    """
    
    pos1 = detector_element_coordinates(detector, element=elem1)
    pos2 = detector_element_coordinates(detector, element=elem2)
    
    distance = np.sqrt(np.sum((pos2 - pos1)**2))
    
    return distance


def detector_element_coordinates(detector, element=None):
    """
    Return the xy coordinates of all detector elements for a given
    system.
    

    Parameters
    ----------
    detector : str
        Detector type: either 'airyscan', 'luminosa', or 'nsparc'
    element : int, optional
        Either None: return all coordinates for all elements
        Or int: return the xy coordinates of a single element only
    
        E.g. detector_element_coordinates('luminosa', element=20) returns
        [0, 0], i.e. the coordinates of the central element

    Returns
    -------
    sx : np.array()
        2D array with xy coordinates

    """
    
    detector = str.lower(detector)
    
    if detector == 'airyscan':
        
        #          22  21
        #     23  10  9   8  20
        #   24  11  2   1   7  19
        #     12  3   0   6  18  31
        #   25  13  4   5  17  30
        #     26 14  15  16  29
        #          27  28
    
        n_elements= 32
        
        shiftx = np.cos(30*np.pi/180)
        sx = np.zeros((n_elements,2))
        sx[0] = [0,0]
    
        sx[1] = [shiftx,1.5]
        sx[2] = [-shiftx,1.5]
        sx[3] = [-2*shiftx,0]
        sx[4] = [-shiftx,-1.5]
        sx[5] = [shiftx,-1.5]
        sx[6] = [2*shiftx,0]
    
        sx[7] = [3*shiftx,1.5]
        sx[8] = [2*shiftx,3]
        sx[9] = [0,3]
        sx[10] = [-2*shiftx,3]
        sx[11] = [-3*shiftx,1.5]
        sx[12] = [-4*shiftx,0]
        sx[13] = [-3*shiftx,-1.5]
        sx[14] = [-2*shiftx,-3]
        sx[15] = [0,-3]
        sx[16] = [2*shiftx,-3]
        sx[17] = [3*shiftx,-1.5]
        sx[18] = [4*shiftx,0]
    
        sx[19] = [5*shiftx,1.5]
        sx[20] = [4*shiftx,3]
        sx[21] = [shiftx,4.5]
        sx[22] = [-shiftx,4.5]
        sx[23] = [-4*shiftx,3]
        sx[24] = [-5*shiftx,1.5]
        sx[25] = [-5*shiftx,-1.5]
        sx[26] = [-4*shiftx,-3]
        sx[27] = [-shiftx,-4.5]
        sx[28] = [shiftx,-4.5]
        sx[29] = [4*shiftx,-3]
        sx[30] = [5*shiftx,-1.5]
        sx[31] = [6*shiftx,0]
        
        sx /= np.sqrt(3)
        
    elif detector in {"luminosa", "pda-23", "pda23"}:
        
        # 31  30  29  28  27
        #   26  25  24  23
        # 22  21  20  19  18
        #   17  16  15  14
        # 13  12  11  10  09
    
        n_elements= 32
        shiftx = np.cos(30*np.pi/180)
        
        sxx  = [4.0, 2.0, 0.0, -2.0, -4.0, 3.0, 1.0, -1.0, -3.0, 4.0,
               2.0, 0.0, -2.0, -4.0, 3.0, 1.0, -1.0, -3.0, 4.0, 2.0,
               0.0, -2.0, -4.0]
        
        syy = [-4*shiftx, -4*shiftx, -4*shiftx, -4*shiftx, -4*shiftx, -2*shiftx, -2*shiftx, -2*shiftx,
              -2*shiftx, 0.0, 0.0, 0.0, 0.0, 0.0, 2*shiftx, 2*shiftx, 2*shiftx, 2*shiftx,
              4*shiftx, 4*shiftx, 4*shiftx, 4*shiftx, 4*shiftx]
      
        
        sx = np.zeros((n_elements,2))
        sx[9:,0] = sxx
        sx[9:,1] = syy
        sx /= 2
        
    elif detector == 'nsparc' or detector == '5x5':
        
        #  0  1  2  3  4
        #  5  6  7  8  9
        # 10 11 12 13 14
        # 15 16 17 18 19
        # 20 21 22 23 24

        n_elements= 25

        sx = np.zeros((n_elements, 2))
        sx[:,0] = [np.mod(i, 5) for i in range(n_elements)]
        sx[:,1] = [i//5 for i in range(n_elements)]
        sx -= 2
        sx[:,1] *= -1

    else:
        return None
    
    if element is None:
        return sx
    else:
        return sx[element]
    